@@ -443,7 +443,7 @@ region_log_stats(region_type *region)
 	char buf[10240], *str=buf;
 	int strl = sizeof(buf);
 	int len=0;
-	snprintf(str, strl, "%lu objects (%lu small/%lu large), %lu bytes allocated (%lu wasted) in %lu chunks, %lu cleanups, %lu in recyclebin%n",
+	len=snprintf(str, strl, "%lu objects (%lu small/%lu large), %lu bytes allocated (%lu wasted) in %lu chunks, %lu cleanups, %lu in recyclebin",
 		(unsigned long) (region->small_objects + region->large_objects),
 		(unsigned long) region->small_objects,
 		(unsigned long) region->large_objects,
@@ -451,8 +451,7 @@ region_log_stats(region_type *region)
 		(unsigned long) region->unused_space,
 		(unsigned long) region->chunk_count,
 		(unsigned long) region->cleanup_count,
-		(unsigned long) region->recycle_size,
-		&len);
+		(unsigned long) region->recycle_size);
 	str+=len;
 	strl-=len;
 	if(1 && region->recycle_bin) {
@@ -466,8 +465,7 @@ region_log_stats(region_type *region)
 				el = el->next;
 			}
 			if(i%ALIGNMENT == 0 && i!=0) {
-				snprintf(str, strl, " %lu%n", (unsigned long)count,
-					&len);
+				len=snprintf(str, strl, " %lu", (unsigned long)count);
 				str+=len;
 				strl-=len;
 			}
